// toolbar.js by Jonathan Le

/**
 * Toolbar
 *
 * @author Jonathan Le
 */
function Toolbar(aParent)
{
	// declare private members
	this.prv = {
		mContent : new Array(),
		mDOM : getTemplate("toolbar").cloneNode(true)
	}; Function.decl.apply(this);
	
	// check the dom
	if (!this.prv.mDOM) {
		delete this.prv;
		return null;
	}
	
	// set the parent
	this.prv.mDOM.parent = this;
	probe(this.prv.mDOM);
	//this.prv.mDOM.id = "toolbar"+getElementsByClassName(document.body, "toolbar");
	generateUniqueId(this.prv.mDOM);
	
	
	this.addButton = function(aLabel)
	{
		var button = new ToolbarButton(aLabel);
		this.addContent(button.prv.mDOM);
		return button;
	};
	
	this.addContent = function(aDOM)
	{
		this.prv.mContent.push(aDOM);
		this.prv.mDOM.contents.insertBefore(aDOM, null);
	};
	
	this.removeContent = function(aDOM)
	{
		for (var i = 0; i < this.prv.mContent.length; i++) {
			if (aDOM == this.prv.mContent[i]) {
				this.prv.mContent.splice(i, 1);
				return this.prv.mDOM.removeChild(aDOM);
			}
		}
	};
	
	// construct
	{
		this.prv.mDOM.style.display = "block";
		// add toolbar to specified div
		if (aParent)
			aParent.appendChild (this.prv.mDOM);
	}

	return this;
}

/**
 * ToolbarButton
 *
 * @author Jonathan Le
 */
function ToolbarButton(aLabel)
{
	this.prv = {
		mLabel : null,
		mDOM : getTemplate("toolbarbutton").cloneNode(true),
		mDisabled : false
	}; Function.decl.apply(this);
	
	// check the dom
	if (!this.prv.mDOM) {
		delete this.prv;
		return null;
	}

	// set the parent, probe
	this.prv.mDOM.parent = this;
	probe(this.prv.mDOM);
	//this.prv.mDOM.id = "toolbarbutton"+getElementsByClassName(document.body, "toolbarbutton");
	generateUniqueId(this.prv.mDOM);	
	
	this.enable = function()
	{
		this.prv.mDisabled = false;
		this.prv.mDOM.className = 'toolbarbutton';
	}

	this.disable = function()
	{
		this.prv.mDisabled = true;
		this.prv.mDOM.className = 'toolbarbutton toolbarbuttondisabled';
		
		this.prv.mDOM.l.className = "l";
		this.prv.mDOM.m.className = "m";
		this.prv.mDOM.r.className = "r";		
	}
	
	this.setLabel = function(aStr)
	{
		// set the internal label
		this.prv.mLabel = aStr;
		this.prv.mDOM.m.label.innerHTML = aStr;
	}
	
	this.prv.mDOM.onmouseover = function(aEvent)
	{

		if (this.parent.prv.mDisabled)
			return false;
		
		var tbtn = this.parent;

		this.l.className = "l_over";
		this.m.className = "m_over";
		this.r.className = "r_over";
	};
	
	this.prv.mDOM.onmouseout = function(aEvent)
	{
		this.l.className = "l";
		this.m.className = "m";
		this.r.className = "r";
	};
	
	this.prv.mDOM.onmousedown = function(aEvent)
	{

		if (this.parent.prv.mDisabled)
			return false;

		this.l.className = "l_down";
		this.m.className = "m_down";
		this.r.className = "r_down";
		
		return false;
	};
	
	this.prv.mDOM.onmouseup = function(aEvent)
	{

		if (this.parent.prv.mDisabled)
			return false;	

		this.l.className = "l_over";
		this.m.className = "m_over";
		this.r.className = "r_over";

	};
	
	this.prv.mDOM.onclick = function(aEvent)
	{

		if (this.parent.prv.mDisabled)
			return false;
	
		var tbtn = this.parent;
		tbtn.onclick(aEvent);
	};
	
	// user definable
	this.onclick = function() {};
	
	// construct here
	{
		this.setLabel(aLabel);
		this.prv.mDOM.style.display = "block";
	}
	
	return this;
}